package fire;

// Fire model for SFI Graduate Workshop in Computational Economics
// (c)2003 Laszlo Gulyas, Scott Moser, Jason Woodard

import java.awt.*;

public class DirVector {

    static final DirVector NULL = new DirVector(0, 0);

    public double x, y;

    public DirVector(Point from, Point to) {
        x = to.getX() - from.getX();
        y = to.getY() - from.getY();
    }

    public DirVector(double x, double y) {
        this.x = x;
        this.y = y;
    }

    public DirVector(double degrees) {
        double theta = Math.toRadians(degrees);
        x = Math.cos(theta);
        y = Math.sin(theta);
    }

    public DirVector add(DirVector vector) {
        return new DirVector(x + vector.x, y + vector.y);
    }

    public DirVector scale(double factor) {
        return new DirVector(x * factor, y * factor);
    }

    public DirVector rotate(double degrees) {
        double theta = Math.toRadians(degrees);
        double newX = Math.cos(theta) * x - Math.sin(theta) * y;
        double newY = Math.sin(theta) * x + Math.cos(theta) * y;
        return new DirVector(newX, newY);
    }

    public Point getMoveFrom(Point from) {

        if (this.getLength() < 0.1) {
            return from;
        }

        int[] diffX = new int[]{1, 1, 0, -1, -1, -1, 0, 1};
        int[] diffY = new int[]{0, 1, 1, 1, 0, -1, -1, -1};

        double d = (this.getAngle() + 22.5) % 360;
        int i = (int)Math.floor(d / 45.0);

        int newX = (int)from.getX() + diffX[i];
        int newY = (int)from.getY() + diffY[i];

        return new Point(newX, newY);
    }

    public double getLength() {
        return Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2));
    }

    public double getAngle() {
        if (this.getLength() < 0.1) return 0;

        double angle = Math.toDegrees(Math.atan2(y, x));
        while (angle < 0) {
            angle += 360;
        }

        return angle % 360;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof DirVector)) return false;

        final DirVector dirVector = (DirVector)o;

        if (x != dirVector.x) return false;
        if (y != dirVector.y) return false;

        return true;
    }

    public int hashCode() {
        int result;
        long temp;
        temp = Double.doubleToLongBits(x);
        result = (int)(temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(y);
        result = 29 * result + (int)(temp ^ (temp >>> 32));
        return result;
    }

    public String toString() {
        return new String("(" + x + ", " + y + ")");
    }


    public static void main(String[] args) {

        // test methods here
        Point p1 = new Point(1, 1);
        Point p2 = new Point(2, 3);
        Point p3 = new Point(-1, -2);

        DirVector v1 = new DirVector(p1, p2);
        DirVector v2 = new DirVector(p2, p3);
        System.out.println("v1 pre: " + v1);
        System.out.println("v2 pre: " + v2);

        DirVector vP = v1.add(v2);
        System.out.println("v1 post: " + v1);
        System.out.println("v2 post: " + v2);
        System.out.println("vP: " + vP);

        DirVector vR = new DirVector(90);
        System.out.println("vR: " + vR);
        System.out.println("length is (1):" + vR.getLength());

        DirVector vRR = vR.rotate(90);
        DirVector vRS = vR.scale(2);

        System.out.println("vRR:" + vRR);
        System.out.println("vRS:" + vRS);

        DirVector vN = new DirVector(0, 0);
        System.out.println("vN:" + vN);
        System.out.println("length is (0): " + vN.getLength());

        Point m1 = vR.getMoveFrom(p1);
        System.out.println("testing move from p1 (1, 2): " + m1);
    }
}
