package fire;

// Fire model for SFI Graduate Workshop in Computational Economics
// (c)2003 Laszlo Gulyas, Scott Moser, Jason Woodard

import uchicago.src.sim.analysis.*;
import uchicago.src.sim.engine.*;
import uchicago.src.sim.gui.*;
import java.util.ArrayList;
import java.util.StringTokenizer;

public class ModelGUI extends Model {

    DisplaySurface dsurf;
    OpenSequenceGraph stats;

    public ModelGUI() {
        Controller.ALPHA_ORDER = false;     	// Non-alphabetic order of probed variables
    }

    public void setup() {
        super.setup();

        name = "FIRE!!!!!!";

        params = new String[]{
            "numAgents",
            "worldSize",
            "emergencyVision",
            "minMoveVision",
            "maxMoveVision",
            "doors",
            "fire",
            "fireGrowthRate"
        };

        if (dsurf != null) {
            dsurf.dispose();
            dsurf = null;
        }

        dsurf = new DisplaySurface(this, "World");
        registerDisplaySurface("Main", dsurf);

        if (stats != null) {
            stats.dispose();
            stats = null;
        }
    }

    public void buildModel() {
        super.buildModel();

        Object2DDisplay display = new Object2DDisplay(world);
        dsurf.addDisplayableProbeable(display, "Display");
        addSimEventListener(dsurf);
        dsurf.display();

        stats = new OpenSequenceGraph("Statistics", this);
        stats.setYRange(0, numAgents);
        stats.setYAutoExpand(false);
        stats.addSequence("#Escaped", new Sequence() {
            public double getSValue() {
                return ModelGUI.this.numEscaped;
            }
        }
        );
        stats.addSequence("#Dead", new Sequence() {
            public double getSValue() {
                return ModelGUI.this.numDead;
            }
        }
        );
        stats.display();
    }

    public void postStep() {
        super.postStep();
        dsurf.updateDisplay();
        stats.step();
    }

    ////////////////////////////////////////////////////////////////////////////////

    public int getNumAgents() {
        return numAgents;
    }

    public void setNumAgents(int a) {
        numAgents = a;
    }

    public int getWorldSize() {
        return worldSize;
    }

    public void setWorldSize(int a) {
        worldSize = a;
    }

    public int getEmergencyVision() {
        return emergencyVision;
    }

    public void setEmergencyVision(int a) {
        emergencyVision = a;
    }

    public int getMinMoveVision() {
        return minMoveVision;
    }

    public void setMinMoveVision(int a) {
        minMoveVision = a;
    }

    public int getMaxMoveVision() {
        return maxMoveVision;
    }

    public void setMaxMoveVision(int a) {
        maxMoveVision = a;
    }

    public String getDoors() {
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < doors.length; i++) {
            sb.append(doors[i][0]);
            sb.append(' ');
            sb.append(doors[i][1]);
            sb.append(' ');
        }

        return sb.toString();
    }

    public void setDoors(String s) {
        StringTokenizer st = new StringTokenizer(s);

        ArrayList list = new ArrayList();

        while (st.hasMoreTokens()) {
            list.add(st.nextToken());
        }

        int num = list.size();
        if (num % 2 == 1) {
            list.add("0");
            num++;
        }

        doors = new int[num / 2][2];
        for (int i = 0; i < num / 2; i++) {
            try {
                String z = (String)list.get(0);
                doors[i][0] = Integer.parseInt(z);
                z = (String)list.get(1);
                doors[i][1] = Integer.parseInt(z);
            } catch (NumberFormatException nfe) {

            }
            list.remove(0);
            list.remove(0);
        }
    }

    public String getFire() {
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < fire.length; i++) {
            sb.append(fire[i][0]);
            sb.append(' ');
            sb.append(fire[i][1]);
            sb.append(' ');
        }

        return sb.toString();
    }

    public void setFire(String s) {
        StringTokenizer st = new StringTokenizer(s);

        ArrayList list = new ArrayList();

        while (st.hasMoreTokens()) {
            list.add(st.nextToken());
        }

        int num = list.size();
        if (num % 2 == 1) {
            list.add("0");
            num++;
        }

        fire = new int[num / 2][2];
        for (int i = 0; i < num / 2; i++) {
            try {
                String z = (String)list.get(0);
                fire[i][0] = Integer.parseInt(z);
                z = (String)list.get(1);
                fire[i][1] = Integer.parseInt(z);
            } catch (NumberFormatException nfe) {

            }
            list.remove(0);
            list.remove(0);
        }
    }

    public int getFireGrowthRate() {
        return fireGrowthRate;
    }

    public void setFireGrowthRate(int a) {
        fireGrowthRate = a;
    }

    ////////////////////////////////////////////////////////////////////////////////

    public static void main(String[] args) {
        SimInit init = new SimInit();
        init.loadModel(new ModelGUI(), "", false);
    }
}